# -*- coding: utf-8 -*-

import json
import gpxpy
import logging
from django.db import transaction
from django.core import serializers
from django.contrib.gis.geos import Point, GEOSGeometry
from cesium.models import GPX_TRACK
from cesium.models import GPX_TRACK_POINTS
logger = logging.getLogger(__name__)

# taken from pyproj
def get_utm_zone(lat, lon):

    try:
        # make sure the longitude is between -180.00 .. 179.9
        valLon = (lon + 180) - int((lon + 180) / 360) * 360 - 180

        # calculate the zone number
        zoneNum = int((valLon + 180) / 6) + 1

        if lat >= 56.0 and lat < 64.0 and valLon >= 3.0 and valLon < 12.0:
            zoneNum = 32

        # special zones for Svalbard
        if lat >= 72.0 and lat < 84.0:
          if valLon >= 0.0 and valLon < 9.0:
              zoneNum = 31;
          elif valLon >= 9.0 and valLon < 21.0:
              zoneNum = 33;
          elif valLon >= 21.0 and valLon < 33.0:
              zoneNum = 35;
          elif valLon >= 33.0 and valLon < 42.0:
              zoneNum = 37;

        if lat < 0:
            epsg = int('327{0}'.format(zoneNum))
        else:
            epsg = int('326{0}'.format(zoneNum))

        logger.info("Track distance will be calculated in EPSG:{0}".format(epsg))

    except Exception as e:
        logger.error("Can't determine EPSG code")

    return epsg

#@transaction.atomic
@transaction.commit_manually
def upload_gpx(request):

    trackType = request.POST.get('upload-radio', '')
    nameItem = request.POST.get('upload-namefield', '')
    commentItem = request.POST.get('upload-commentfield', '')
    fileItem = request.FILES.get('upload-filefield', '')
    user = request.user
    success = 'false'
    message = ''
    czmlString = ''
    lastTrackPoints = GPX_TRACK_POINTS()
    utmZone = None
    len = None

    # Test if the file was uploaded
    if fileItem:
        fn = fileItem.name
        data = fileItem.file.read()

        try:
            gpx = gpxpy.parse(data.decode('UTF-8'), 'lxml')

            # write the gpx track (single row) to the database
            gpxTrack = GPX_TRACK(
                userid = user,
                name = nameItem,
                desc = commentItem,
                type = trackType
            )

            gpxTrack.save()

            # write the track points (multiple rows) to database
            for track in gpx.tracks:
                for segment in track.segments:
                    for point in segment.points:

                        trackPoints = GPX_TRACK_POINTS(
                            trackid = gpxTrack,
                            name = point.name
                                if point.name is not None else None,
                            lon = point.longitude
                                if point.longitude is not None else None,
                            lat = point.latitude
                                if point.latitude is not None else None,
                            elv = point.elevation
                                if point.elevation is not None else None,
                            time = point.time
                                if point.time is not None else None,
                            cmt = point.comment
                                if point.comment is not None else None,
                            speed = point.speed
                                if point.speed is not None else None,
                            sym = point.symbol
                                if point.symbol is not None else None,
                            hdop = point.horizontal_dilution
                                if point.horizontal_dilution is not None else None,
                            vdop = point.vertical_dilution
                                if point.vertical_dilution is not None else None,
                            pdop = point.position_dilution
                                if point.position_dilution is not None else None,
                            geom = GEOSGeometry("POINT({0} {1})".format(point.longitude, point.latitude))
                                if all((point.longitude, point.latitude)) else None
                        )

                        if utmZone is None:
                            utmZone = get_utm_zone(point.latitude, point.longitude)

                        # calculate distance for for every point to parent
                        if (lastTrackPoints.dist is not None):
                            currentGeom = trackPoints.geom.transform(utmZone, clone=True)
                            lastGeom = lastTrackPoints.geom.transform(utmZone, clone=True)
                            d = currentGeom.distance(lastGeom)
                            len = lastTrackPoints.dist + d
                            trackPoints.dist = len
                        else:
                            trackPoints.dist = 0

                        trackPoints.save()

                        lastTrackPoints = trackPoints

            # write the track length
            gpxTrack.length = len
            gpxTrack.save()

            transaction.commit()

            message = 'Die Datei ' + fn + ' wurde erfolgreich hochgeladen'
            success = 'true'

            retMsg = '{"success": ' + str(success) + ', '\
                '"msg":"' + message + '", '\
                '"trkid":' + str(gpxTrack.id) +'}'

        except gpxpy.gpx.GPXException as e:
            message = 'Die Datei ' + fn + ' konnte nicht gelesen werden: ' + str(e)
            success = 'false'

            retMsg = '{"success": ' + str(success) + ', '\
                '"msg":"' + message + '"}'

    else:
        message = 'Upload fehlgeschlagen'
        success = 'false'

    return retMsg

if __name__ == '__main__':
    upload_gpx(request)