/*global Ext, OpenLayers*/
/*jslint plusplus: true, sloppy: true, white: true*/

/**


 Published under the BSD license.

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 Neither the name of the terrestris GmbH & Co. KG nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

 @author terrestris GmbH & Co. KG
 */

/**
 * A config parser to create GIS-Client related objects
 * from Application Context
 */
Ext.define('ExtACe.parser.Config', {
    statics: {
        /**
         * Create cesium layer objects from conf
         */
        createImageryProvider: function(layersArr) {
            var me = this,
                layers = layersArr || [],
                provider,
                providersArr = [];

            Ext.each(layers, function(layer) {
                switch(layer.imageryProvider) {
                    case 'ArcGisMapServer':
                        provider = new Cesium.ArcGisMapServerImageryProvider({
                            url: layer.url,
                            tileDiscardPolicy: layer.tileDiscardPolicy,
                            proxy: layer.proxy,
                            usePreCachedTilesIfAvailable: layer.usePreCachedTilesIfAvailable
                        });
                        provider.name = layer.name;
                        break;
                    case 'BingMaps':
                        provider = new Cesium.BingMapsImageryProvider({
                            url: layer.url || 'http://dev.virtualearth.net',
                            key: layer.key || undefined,
                            mapStyle: me.getBingMapStyle(layer.mapStyle) || Cesium.BingMapsStyle.AERIAL,
                            tileDiscardPolicy: layer.tileDiscardPolicy || undefined,
                            proxy: layer.proxy ? new Cesium.DefaultProxy(layer.proxy) : undefined
                        });
                        provider.name = layer.name;
                        break;
                    case 'OpenStreetMap':
                        provider = new Cesium.OpenStreetMapImageryProvider({
                            url: layer.url || 'http://tile.openstreetmap.org',
                            fileExtension: layer.fileExtension || 'png',
                            proxy: layer.proxy ? new Cesium.DefaultProxy(layer.proxy) : undefined,
                            extent: layer.extent || new Cesium.Extent(-3.141592653589793, -1.4844222297453324, 3.141592653589793, 1.4844222297453322),
                            maximumLevel: layer.maximumLevel || 18,
                            credit: layer.credit || 'MapQuest, Open Street Map and contributors, CC-BY-SA'
                        });
                        provider.name = layer.name;
                        break;
                    case 'SingleTile':
                        provider = new Cesium.SingleTileImageryProvider({
                            url: layer.url,
                            extent: layer.extent,
                            proxy: layer.proxy,
                            credit: layer.credit
                        });
                        provider.name = layer.name;
                        break;
                    case 'TileMapService':
                        provider = new Cesium.TileMapServiceImageryProvider({
                            url: layer.url,
                            fileExtension: layer.fileExtension,
                            proxy: layer.proxy,
                            credit: layer.credit,
                            minimumLevel: layer.minimumLevel,
                            maximumLevel: layer.maximumLevel,
                            extent: layer.extent,
                            tilingScheme: layer.tilingScheme,
                            tileWidth: layer.tileWidth,
                            tileHeight: layer.tileHeight
                        });
                        provider.name = layer.name;
                        break;
                    case 'WebMapService':
                        provider = new Cesium.WebMapServiceImageryProvider({
                            url: layer.url,
                            layers: layer.layers,
                            parameters: layer.parameters,
                            extent: layer.extent || new Cesium.Extent(-3.141592653589793, -1.4844222297453324, 3.141592653589793, 1.4844222297453322),
                            maximumLevel: layer.maximumLevel || 18,
                            credit: layer.credit,
                            proxy: {
                                getURL : function(url) {
                                    return layer.proxy + encodeURIComponent(url);
                                }
                            }
                        });
                        provider.name = layer.name;
                        break;
                }

                providersArr.push(provider)

            });

            return providersArr;
        },

        /**
         *
         */
        createViewer: function(conf) {
            var me = this,
                userConf = conf.user,
                viewerConf = conf.viewer,
                layerConf = conf.layers;

            // create the viewer and render it to the cesium container div
            var viewer = new Cesium.Viewer(viewerConf.containerId, {
                animation: viewerConf.animation,
                baseLayerPicker: viewerConf.baseLayerPicker,
                fullscreenButton: true,
                homeButton: viewerConf.homeButton,
                geocoder: false,
                imageryProvider: false,
                sceneMode: me.getSceneMode(viewerConf.sceneMode),
                sceneModePicker: viewerConf.sceneModePicker,
                showRenderLoopErrors: viewerConf.showRenderLoopErrors,
                timeline: viewerConf.timeline,
                terrainProvider: new Cesium.VRTheWorldTerrainProvider({
                    url : 'http://www.vr-theworld.com/vr-theworld/tiles1.0.0/73/',
                    credit : 'Terrain data courtesy VT MÄK'
                }),
                useDefaultRenderLoop: viewerConf.useDefaultRenderLoop
            });

//            var scene = viewer.scene;

//            // zoom to custom extent
//            var destination = Cesium.Cartographic.fromDegrees(13.353278, 52.544482, 2000.0);
//
//            var flight = Cesium.CameraFlightPath.createAnimationCartographic(scene, {
//                destination : destination
//            });
//            scene.getAnimations().add(flight);

            //Allow users to zoom and follow objects loaded from CZML by clicking on it.
            viewer.extend(Cesium.viewerDynamicObjectMixin);

            // get the layers object
            var layers = viewer.centralBody.getImageryLayers();

            // create layers and append it to the viewer
            Ext.Array.each(me.createImageryProvider(layerConf), function(provider) {
                var layer = layers.addImageryProvider(provider);
                layer.name = provider.name;
                layer.visibility = true;
            });

            return viewer;
        },

        /**
         *
         */
        getSceneMode: function(sceneMode) {
            var me = this;

            switch (sceneMode) {
                case '2D':
                    return Cesium.SceneMode.SCENE2D;
                case '2.5D':
                    return Cesium.SceneMode.COLUMBUS_VIEW;
                case '3D':
                    return Cesium.SceneMode.SCENE3D;
            }

        },

        /**
         *
         */
        getBingMapStyle: function(style) {
            var me = this;

            switch (style) {
                case 'AERIAL':
                    return Cesium.BingMapsStyle.AERIAL;
                case 'AERIAL_WITH_LABELS':
                    return Cesium.BingMapsStyle.AERIAL_WITH_LABELS;
                case 'COLLINS_BART':
                    return Cesium.BingMapsStyle.COLLINS_BART;
                case 'ORDNANCE_SURVEY':
                    return Cesium.BingMapsStyle.ORDNANCE_SURVEY;
                case 'ROAD':
                    return Cesium.BingMapsStyle.ROAD;
            }

        },

        /**
         * Sets up the GUI in reaction to the loginstatus
         */
        setupGUI: function(user) {
            var uploadButton = Ext.ComponentQuery.query('t_gpx_upload_button')[0],
                loginButton = Ext.ComponentQuery.query('t_login_button')[0],
                registrationButton = Ext.ComponentQuery.query('t_registration_button')[0],
                controlcenterButton = Ext.ComponentQuery.query('t_controlcenter_button')[0];

            if (user){
                uploadButton.enable();
                loginButton.hide();
                registrationButton.hide();

                controlcenterButton.setText(user.name);
                controlcenterButton.show();
            }
            else{
                loginButton.show();
                registrationButton.show();
                controlcenterButton.hide();
            }
        }

    }
});